@@ -1,4 +1,4 @@
-/*	$Id: vilter-attachment.c,v 1.48 2006/02/09 11:17:39 mbalmer Exp $	*/
+/*	$Id: vilter-attachment.c,v 1.55 2006/02/17 07:33:14 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2004, 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -50,9 +50,11 @@
 #include <sys/syslimits.h>
 #endif
 
+extern char *tmpdir;
+
 char *attachment_notification;
 
-SLIST_HEAD(, part) dhead;
+SLIST_HEAD(parthead, part);
 SLIST_HEAD(argshead, arg);
 
 struct pat_head fname_pats;
@@ -86,7 +88,7 @@ find_base64(char c)
 
 	for (r = 0; base64_chars[r] != '\0' && base64_chars[r] != c; r++)
 		;	
-	return (char) r; 
+	return (char)r; 
 }
 
 static int
@@ -128,7 +130,7 @@ mstrlen(const char *p, const char *eom)
 }
 
 static char *
-analyze_header(struct be_data *priv, char *hdr, char *eom, int *name_len, struct argshead *arguments)
+analyze_header(char *hdr, char *eom, int *name_len, struct argshead *arguments)
 {
 	char *value;
 	int value_len;
@@ -180,10 +182,9 @@ analyze_header(struct be_data *priv, char *hdr, char *eom, int *name_len, struct
 		while (p < eov && (*p == ' ' || *p == '\t'))
 			++p;
 
-		a = (struct arg *) malloc(sizeof(struct arg));
+		a = (struct arg *)malloc(sizeof(struct arg));
 
 		if (a == NULL) {
-			warnx("attachment: memory allocation failure");
 			syslog(LOG_ERR, "attachment: unable to allocate memory for header");
 			/* XXX if this happens we crash */
 		}
@@ -230,7 +231,7 @@ analyze_header(struct be_data *priv, char *hdr, char *eom, int *name_len, struct
 
 
 static int
-analyze_part(struct be_data *priv, struct part *p, char *eom, char *pattern, int patlen)
+analyze_part(struct part *p, char *eom, char *pattern, size_t patlen)
 {
 	int inhdr;
 	char *m;
@@ -277,7 +278,7 @@ analyze_part(struct be_data *priv, struct part *p, char *eom, char *pattern, int
 					if (decoded[0] == 0xff && decoded[1] == 0xd8) {
 						warnx("attachment: found a jpeg image");
 						p->skip = skip = 1;
-						strncpy(pattern, "JPEG IMAGE", patlen);
+						strlcpy(pattern, "JPEG IMAGE", patlen);
 					}
 				}
 			}
@@ -287,7 +288,7 @@ analyze_part(struct be_data *priv, struct part *p, char *eom, char *pattern, int
 			SLIST_INIT(&arguments);
 			hdr = m;
 
-			m = analyze_header(priv, hdr, eom, &name_len, &arguments);
+			m = analyze_header(hdr, eom, &name_len, &arguments);
 			/* mprint(hdr, name_len, eom); */
 
 			if (name_len >= strlen("Content-Type") && !strncmp(hdr, "Content-Type", strlen("Content-Type"))) {
@@ -362,13 +363,14 @@ analyze_part(struct be_data *priv, struct part *p, char *eom, char *pattern, int
 }
 
 int
-vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namelen)
+vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 {
 	char *msg;
 	int fd;
 	struct stat statbuf;
 	struct part *p;
 	struct part *lp;
+	struct parthead parts;
 	int lines;
 	char *m, *q;
 	char *som;	/* Start of message, points to the first byte of the mmapped file */
@@ -384,11 +386,13 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 	FILE *fp;
 	char buf[1024];
 	int nread;
+	int retval;
 
 	p = lp = NULL;
 	boundary = NULL;
 	lines = skip = boundary_len = 0;
 	inhdr = 1;
+	retval = SCAN_OK;
 
 	if (SLIST_EMPTY(&fname_pats) && SLIST_EMPTY(&ctype_pats))
 		return SCAN_OK;
@@ -409,9 +413,9 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 		return SCAN_ERROR;
 	}
 
-	SLIST_INIT(&dhead);
+	SLIST_INIT(&parts);
 
-	size = (long) statbuf.st_size;
+	size = (long)statbuf.st_size;
 	som = msg;
 	eom = som + size - 1;	/* eom points to the last byte */
 
@@ -473,7 +477,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 
 							if ((p->start = sol(m, eom)) != NULL) {
 								if (lp == NULL)
-									SLIST_INSERT_HEAD(&dhead, p, parts);
+									SLIST_INSERT_HEAD(&parts, p, parts);
 								else
 									SLIST_INSERT_AFTER(lp, p, parts);
 							} else {
@@ -500,15 +504,22 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 	if (p != NULL && p->end == 0)
 		p->end = eom;
 
-	SLIST_FOREACH(p, &dhead, parts)
-		skip += analyze_part(priv, p, eom, virus, namelen);
+	SLIST_FOREACH(p, &parts, parts)
+		skip += analyze_part(p, eom, virus, namelen);
 
 	if (verbose)
 		warnx("attachment: will skip %d attachments", skip);
 
 	if (skip > 0) {
-		tmpnam(newfn);
-		newfd = open(newfn, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
+		strlcpy(newfn, tmpdir, sizeof(newfn));
+		strlcat(newfn, "/vilter-attachment.XXXXXXXXXX", sizeof(newfn));
+		if ((newfd = mkstemp(newfn)) == -1) {
+			syslog(LOG_ERR, "unable to create temp file");
+			retval = SCAN_ERROR;
+			goto endproc;
+		}
+
+
 		write(newfd, "This is a multi-part message in MIME format.\r\n\r\n", strlen("This is a multi-part message in MIME format.\r\n\r\n"));
 		write(newfd, "--", 2);
 		write(newfd, boundary, boundary_len);
@@ -522,7 +533,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 		} else
 			write(newfd, "Vaporized an unwelcome attachment\r\n\r\n", strlen("Vaporized an unwelcome attachment\r\n\r\n"));
 
-		SLIST_FOREACH(p, &dhead, parts) {
+		SLIST_FOREACH(p, &parts, parts) {
 			if (p->skip == 0 && (p->end - p->start) > 0) {
 				write(newfd, "--", 2);
 				write(newfd, boundary, boundary_len);
@@ -534,9 +545,10 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 		write(newfd, boundary, boundary_len);
 		write(newfd, "--\r\n", 4);
 		close(newfd);
-		strcpy(fn, newfn);
+		strlcpy(fn, newfn, fnlen);
 	}
 
+endproc:
 	if (munmap(msg, statbuf.st_size)) {
 		syslog(LOG_ERR, "attachment: can't munmap file");
 		/* memory leak */
@@ -544,10 +556,10 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 	}
 
 	close(fd);
-
-	while (!SLIST_EMPTY(&dhead)) {
-		p = SLIST_FIRST(&dhead);
-		SLIST_REMOVE_HEAD(&dhead, parts);
+	
+	while (!SLIST_EMPTY(&parts)) {
+		p = SLIST_FIRST(&parts);
+		SLIST_REMOVE_HEAD(&parts, parts);
 		free(p);
 	}
 
