@@ -1,4 +1,4 @@
-/*	$Id: vilter-icap.c,v 1.27 2006/02/08 15:50:07 mbalmer Exp $	*/
+/*	$Id: vilter-icap.c,v 1.32 2006/02/17 07:33:14 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2004, 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -57,7 +57,7 @@ int	 icap_tries;
 int	 icap_timeout;
 
 int
-vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namelen)
+vilter_scan(char *fn, size_t fnlen, char *chroot, char *virus, size_t namelen)
 {
 	int fd;
 	int try;
@@ -88,7 +88,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 	}
 
 	if (bind_addr != NULL) {
-		bzero((char *) &server_sockaddr, sizeof(server_sockaddr));
+		bzero((char *)&server_sockaddr, sizeof(server_sockaddr));
 		bind_sockaddr.sin_family = AF_INET;
 
 		if (inet_aton(bind_addr, &bind_sockaddr.sin_addr) == 0) {
@@ -101,13 +101,13 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 			bind_sockaddr.sin_family = hostent->h_addrtype;
 			memcpy(&bind_sockaddr.sin_addr, hostent->h_addr, hostent->h_length);
 		}
-		if (bind(fd, (struct sockaddr *) &bind_sockaddr, sizeof(bind_sockaddr))) {
+		if (bind(fd, (struct sockaddr *)&bind_sockaddr, sizeof(bind_sockaddr))) {
 			syslog(LOG_ERR, "icap: can't bind to address %s", bind_addr);
 			close(fd);
 			return SCAN_ERROR;
 		}
 	}
-	bzero((char *) &server_sockaddr, sizeof(server_sockaddr));
+	bzero((char *)&server_sockaddr, sizeof(server_sockaddr));
 
 	server_sockaddr.sin_family = AF_INET;
 	server_sockaddr.sin_port = htons(icap_port);
@@ -124,7 +124,7 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 	}
 	try = 0;
 
-	while (connect(fd, (struct sockaddr *) &server_sockaddr, sizeof(server_sockaddr)) == -1) {
+	while (connect(fd, (struct sockaddr *)&server_sockaddr, sizeof(server_sockaddr)) == -1) {
 		++try;
 
 		if (try == icap_tries) {
@@ -178,8 +178,11 @@ vilter_scan(struct be_data *priv, char *fn, char *chroot, char *virus, int namel
 	}
 	fp = fopen(fn, "r");
 
-	if (fp == NULL)
-		warnx("icap: can't open file");
+	if (fp == NULL) {
+		syslog(LOG_ERR, "icap: can't open file");
+		close(fd);
+		return SCAN_ERROR;
+	}
 
 	bzero(buf, sizeof(buf));
 
