@@ -1,4 +1,4 @@
-/*	$Id: smtp-vilter.h,v 1.48 2006/02/08 15:50:07 mbalmer Exp $	*/
+/*	$Id: smtp-vilter.h,v 1.54 2006/02/16 08:50:57 mbalmer Exp $	*/
 
 /*
  * Copyright (c) 2003, 2004, 2005, 2006 Marc Balmer <marc@msys.ch>
@@ -22,9 +22,9 @@
 #include <stdio.h>
 #include <unistd.h>
 
-#define	VERSION	"1.3.0"
+#define	VERSION	"1.3.1"
 
-#define MAJOR	5	/* Major and minor version number */
+#define MAJOR	7	/* Major and minor version number */
 #define MINOR	1	/* of shared libs (backends) */
 
 #if __OpenBSD__
@@ -68,11 +68,6 @@ enum strategies {
 #define BE_SCAN_UNWANTED	0x0004
 #define BE_SCAN_PER_RCPT	0x0008
 
-struct be_data {
-	struct syslog_data	*sdata;
-	void			*priv;
-};
-
 struct backend {
 	char	*name;
 	char	*config_file;
@@ -81,13 +76,12 @@ struct backend {
 	int	 (*be_init)(char *);	/* One time initialization */
 	char	*(*be_name)(void);
 	int	 (*be_type)(void);
-	int	 (*be_new)(struct be_data *, char *, FILE *);	/* Per thread initializer */
+	int	 (*be_envfrom)(char *, FILE *);
 	union {
-		int (*be_scan_virus)(struct be_data *, char *, char *, char *, int);
-		int (*be_scan_spam)(struct be_data *, char *, char *, double *, double *);
-		int (*be_scan_unwanted)(struct be_data *, char *, char *, char *, int);
+		int (*be_scan_virus)(char *, size_t, char *, char *, size_t);
+		int (*be_scan_spam)(char *, size_t, char *, double *, double *);
+		int (*be_scan_unwanted)(char *, size_t, char *, char *, size_t);
 	} be_scan;
-	int	 (*be_end)(struct be_data *priv);	/* Per thread finalizer */
 	void	 (*be_exit)(void);
 };
 
